/*
 * Copyright (c) 2009 The Trustees of Indiana University and Indiana
 *                    University Research and Technology
 *                    Corporation.  All rights reserved.
 *
 * Author(s): Torsten Hoefler <htor@cs.indiana.edu>
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mpiparmetis.hpp"
#include <mpi.h>
#include <parmetis.h>

int main() {

  MPI_Init(NULL, NULL);
  int r, p;
  MPI_Comm comm=MPI_COMM_WORLD;
  MPI_Comm_rank(comm, &r);
  MPI_Comm_size(comm, &p);

  /* generate example graph */ 
  int nvert = 10; // vertices per process
  int totvert = nvert*p; // total # of vertices
  int degree = 2; // degree per vertex

  int printrank=0; // set to negative to disable

  idxtype *vtxdist = (idxtype*)malloc((p+1)*sizeof(idxtype)); //  how the vertices of the graph are distributed among the processors
  for(int i=0; i<(p+1); i++) if(i==0) vtxdist[i]=0; else vtxdist[i]=vtxdist[i-1]+nvert;
  if(r==printrank) { printf("[%i] vtxdist: ", r); for(int i=0; i<(p+1); i++) printf("%i ", vtxdist[i]); printf("\n"); }
  idxtype *xadj = (idxtype*)malloc((nvert+1)*sizeof(idxtype)); // CSR index
  for(int i=0; i<(nvert+1); i++) if(i==0) xadj[i]=0; else xadj[i]=xadj[i-1]+degree;
  if(r==printrank) { printf("[%i] xadj: ",r); for(int i=0; i<nvert+1; i++) printf("%i ", xadj[i]); printf("\n"); }
  idxtype *adjncy = (idxtype*)malloc(nvert*degree*sizeof(idxtype)); // CSR list
  for(int i=0; i<nvert; i++) for(int j=0; j<degree; j++) { // simple ring (ParMETIS needs an undirected graph :-() 
    int pos=i*degree+j; 
    if(j==0) adjncy[pos]=(r*nvert+i+totvert-1)%totvert; 
    if(j==1) adjncy[pos]=(r*nvert+i+1)%totvert; 
  }
  if(r==printrank) { printf("[%i] adjncy: ",r); for(int i=0; i<nvert*degree; i++) printf("%i ", adjncy[i]); printf("\n"); }

  idxtype *vwgt=NULL; // vertex weights
  idxtype *adjwgt=NULL; // edge weights
  int wgtflag=0; // 0 No weights (vwgt and adjwgt are both NULL).  1 Weights on the edges only (vwgt is NULL).  2 Weights on the vertices only (adjwgt is NULL).  3 Weights on both the vertices and edges.
  int numflag=0; // numbering scheme - 0 C-style numbering that starts from 0. 1 Fortran-style numbering that starts from 1.
  int ncon=0; // number of weights that each vertex has.
  int nparts=p; // number of sub-domains that are desired.
  float *tpwgts = NULL; // specify the fraction of vertex weight that should be distributed to each sub-domain for each balance constraint.
  float *ubvec = NULL; // specify the imbalance tolerance for each vertex weight,
  int options[3] = {0,0,0};
  int edgecut; // the number of edges that are cut by the partitioning is written to this
  idxtype *part = (idxtype*)malloc(nvert*sizeof(idxtype)); // This is an array of size equal to the number of locally-stored vertices. part[i] is the new position of vertex i 
  
  ParMETIS_V3_PartKway(vtxdist, xadj, adjncy, vwgt, adjwgt, &wgtflag, &numflag, &ncon, &nparts, tpwgts, ubvec, options, &edgecut, part, &comm);

  if(r==printrank) { printf("[%i] part: ", r); for(int i=0; i<nvert; i++) printf("%i ", part[i]); printf("\n"); }

  MPI_Comm newcomm;
  MPIX_Graph_create_parmetis_unweighted(vtxdist, xadj, adjncy, &numflag, part, &comm, &newcomm);
  
  MPI_Finalize();

}
